#include "ViewInterpolate.h"


#include "ccGenericGLDisplay.h"

//QT
#include <QString>

//CCLib
#include <CCGeom.h>

ViewInterpolate::ViewInterpolate () :
    p_view1 ( NULL ),
    p_view2 ( NULL ),
    m_total_steps ( 0 ),
    m_current_step (0)
{

}


ViewInterpolate::ViewInterpolate(cc2DViewportObject * a_first_view,  cc2DViewportObject * a_second_view, unsigned int a_max_steps ) :
    p_view1 ( a_first_view ),
    p_view2 ( a_second_view ),
    m_total_steps ( a_max_steps ),
    m_current_step (0)
{

}

ViewInterpolate::~ViewInterpolate()
{
}

void ViewInterpolate::View1 ( cc2DViewportObject * a_first_view )
{
    p_view1 = a_first_view;
}

cc2DViewportObject * ViewInterpolate::View1 ()
{
    return p_view1;
}

void ViewInterpolate::View2 ( cc2DViewportObject * a_second_view )
{
    p_view2 = a_second_view;
}

cc2DViewportObject * ViewInterpolate::View2 ()
{
    return p_view2;
}

bool ViewInterpolate::NextView ( cc2DViewportObject * a_returned_viewport )
{
    if ( ( m_current_step >= m_total_steps ) || ( p_view1 == NULL ) || ( p_view2 == NULL ) )
    {
        return false;
    }

    //initial and final views
    ccViewportParameters view1 ( p_view1->getParameters() );
    ccViewportParameters view2 ( p_view2->getParameters() );
    ccViewportParameters working_view ( p_view1->getParameters() );

    //interpolation fraction
    double interpolate_fraction ( static_cast <double> ( m_current_step ) / static_cast <double> ( m_total_steps ) );

    working_view.pixelSize = InterpolateNumber ( view1.pixelSize, view2.pixelSize, interpolate_fraction );
    working_view.zoom = InterpolateNumber ( view1.zoom, view2.zoom, interpolate_fraction );
    working_view.defaultPointSize = InterpolateNumber ( view1.defaultPointSize, view2.defaultPointSize, interpolate_fraction );
    working_view.defaultLineWidth = InterpolateNumber ( view1.defaultLineWidth, view2.defaultLineWidth, interpolate_fraction );
    working_view.zNearCoef = InterpolateNumber ( view1.zNearCoef, view2.zNearCoef, interpolate_fraction );
    working_view.zNear = InterpolateNumber ( view1.zNear, view2.zNear, interpolate_fraction );
    working_view.zFar = InterpolateNumber ( view1.zFar, view2.zFar, interpolate_fraction );
    working_view.fov = InterpolateNumber ( view1.fov, view2.fov, interpolate_fraction );
    working_view.perspectiveAspectRatio = InterpolateNumber ( view1.perspectiveAspectRatio, view2.perspectiveAspectRatio, interpolate_fraction );
    working_view.orthoAspectRatio = InterpolateNumber ( view1.orthoAspectRatio, view2.orthoAspectRatio, interpolate_fraction );

    ccGLMatrixd view_mat ( working_view.viewMat );
    //
    working_view.viewMat = view_mat;
    for ( unsigned int i = 0 ; i < OPENGL_MATRIX_SIZE ; ++i )
    {
        working_view.viewMat.data()[i] = InterpolateNumber ( view1.viewMat.data()[i], view2.viewMat.data()[i], interpolate_fraction );
    }

    for ( unsigned int i = 0 ; i<3 ; ++i )
    {
         working_view.pivotPoint[i] = InterpolateNumber ( view1.pivotPoint[i], view2.pivotPoint[i], interpolate_fraction );
         working_view.cameraCenter[i] = InterpolateNumber ( view1.cameraCenter[i], view2.cameraCenter[i], interpolate_fraction );
    }

    a_returned_viewport->setParameters( working_view );

    ++m_current_step;

    return true;
}

unsigned int ViewInterpolate::CurrentStep ()
{
    return m_current_step;
}

void ViewInterpolate::CurrentStep ( unsigned int a_current_step )
{
    m_current_step = a_current_step;
}

unsigned int ViewInterpolate::MaxStep()
{
    return m_total_steps;
}

void ViewInterpolate::MaxStep ( unsigned int a_max_step )
{
    m_total_steps = a_max_step;
}
