//##########################################################################
//#                                                                        #
//#                            CLOUDCOMPARE                                #
//#                                                                        #
//#  This program is free software; you can redistribute it and/or modify  #
//#  it under the terms of the GNU General Public License as published by  #
//#  the Free Software Foundation; version 2 of the License.               #
//#                                                                        #
//#  This program is distributed in the hope that it will be useful,       #
//#  but WITHOUT ANY WARRANTY; without even the implied warranty of        #
//#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         #
//#  GNU General Public License for more details.                          #
//#                                                                        #
//#         COPYRIGHT: Ryan Wicks, 2G Robotics Inc., 2015				   #
//#                                                                        #
//##########################################################################


#ifndef VIEWINTERPOLATE_H
#define VIEWINTERPOLATE_H

#include "cc2DViewportObject.h"

class ccViewportParameters;

/**
 * @brief The ViewInterpolate class
 * This class takes pointers to two viewport objects, and returns intermediate viewports between over a set number of steps.
 */
class ViewInterpolate
{
public:
    //constructor
    ViewInterpolate( );
    ViewInterpolate( cc2DViewportObject * a_first_view,  cc2DViewportObject * a_second_view, unsigned int a_max_steps = 0 );
    ~ViewInterpolate();

    //accessor for the first viewport object.
    void View1 ( cc2DViewportObject * a_first_view );
    cc2DViewportObject * View1 ();

    //accessor for the second viewport object
    void View2 ( cc2DViewportObject * a_second_view );
    cc2DViewportObject * View2 ();

    //get the next view
    bool NextView ( cc2DViewportObject * a_returned_viewport );

    //accessors for the current step
    unsigned int CurrentStep ();
    void CurrentStep ( unsigned int a_current_step );

    //accessor for the max number of steps
    unsigned int MaxStep();
    void MaxStep ( unsigned int a_max_step );

private:

    cc2DViewportObject * p_view1;

    cc2DViewportObject * p_view2;

//    cc2DViewportObject m_returned_view;

    unsigned int m_total_steps;

    unsigned int m_current_step;
};

//helper function for interpolating between simple numerical types
template <class T>
T InterpolateNumber ( T start_num, T end_num, double interpolation_fraction )
{
    double start_double ( static_cast <double> ( start_num ) );
    double end_double ( static_cast <double> ( end_num ) );

    return static_cast < T > ( start_double + (end_double - start_double) * interpolation_fraction );
}

#endif // VIEWINTERPOLATE_H
